;---------------------------------------;
; - ovladani dvou DC-elektromotoru      ;
;   (intenzita i smer) pomoci dvou      ;
;   potenciometru                       ;
; - Pouzijte modul budice               ;
;   a textovy LCD displej               ;
;---------------------------------------;
	LIST	P = 16F877, R = DEC
	include	p16f877.inc

	__CONFIG	H'3F3A'

;---------------------------------------

	include	tab_util.inc
	include	div16u.inc
	include	displej.inc
	include	dskripty.inc
	include	ad_prev.inc

OFFRAM	EQU	H'20'	; zacatek volne pameti "RAM"

#DEFINE	DB4	PORTE,2	; vyvody pro ovladani displeje
#DEFINE	DB5	PORTE,1
#DEFINE	DB6	PORTE,0
#DEFINE	DB7	PORTA,4
#DEFINE	E	PORTB,1
#DEFINE	RW	PORTB,2
#DEFINE	RS	PORTB,3

#DEFINE	PWMPIN1	PORTC,2	; prvni pol motoru 1
#DEFINE	PWMPIN2	PORTC,1	; prvni pol motoru 2
#DEFINE	SMER1	PORTC,0	; druhy pol motoru 1
#DEFINE	SMER2	PORTA,5	; druhy pol motoru 2

	CBLOCK	OFFRAM
	TML:1	; pro obsluhu displeje
	TMH:1
	DDATA:1

	AA:2	; pro skripty
	BB:2
	CC:2
	VD16U:1
	NUMFORM_:1

	MTMP:1	; obsluha PWM
	PWM_TEMP:1

	ADTEMP:1	; prevodnik
	ADRESL_:1
	ADRESH_:1
	AD_COUNT:1

	HODNOTA1:1 ; vystupy prevodniku
	HODNOTA2:1
	ENDC

;--------------------------------------

	ORG      0
	NOP	; 3x NOP kvuli zavadeci
	NOP
	NOP

	banksel	TRISA
	BCF	SMER1	; nastaveni ridicich pinu
	BCF	SMER2
	BCF	PWMPIN1
	BCF	PWMPIN2
	MOVLW	B'00000000' ; zakladni nastaveni
	MOVWF	OPTION_REG
	banksel	T2CON	; nastaveni casovace pro PWM (preddelicka = 1:16)
	MOVLW	B'00000111' ; (pro rizeni DC motoru staci nizsi frekvence modulace)
	MOVWF	T2CON
	banksel	PR2
	MOVLW	D'31'	; 7bitovy vystup PWM - perioda TMR2 = 32 (*4)
	MOVWF	PR2
	MOVLW	B'10000100' ; nastaveni prevodniku
	MOVWF	ADCON1
	banksel	ADCON0
	MOVLW	B'10000001'
	MOVWF	ADCON0

	CLRF	TMR2	; nastaveni PWM a vynulovani faze (TMR2)
	CLRF	CCPR1L	; kanal CCP1
	MOVLW	B'00001100'
	MOVWF	CCP1CON
	CLRF	CCPR2L	; kanal CCP2
	MOVLW	B'00001100'
	MOVWF	CCP2CON

	MOVLW	D'128'	; pocatecni ridici hodnoty
	MOVWF	HODNOTA1
	MOVLW	D'128'
	MOVWF	HODNOTA2

	CALL	INITDISP

OPAKUJ	MOVF	HODNOTA1,W ; od ridici hodnoty 1 (prvni kanal)...
	ADDLW	-D'128'	; ...odecteme polovinu celeho bytu (znamenkova konvence)
	CALL	MOTOR1	; vysledkem nastavime intenzitu otaceni motoru 1

	MOVF	HODNOTA2,W ; totez provedeme pro druhy motor
	ADDLW	-D'128'
	CALL	MOTOR2

	MOVLW	D'0'	; vypiseme puvodni ridici hodnoty na displej
	CALL	SKRIPT

	MOVLW	D'3'	; provedeme A/D prevod na kanalu 3 (potenciometr P2)
	CALL	PREVOD
	banksel	ADRESH
	RRF	ADRESH,F
	banksel	ADRESL
	RRF	ADRESL,F
	banksel	ADRESH
	RRF	ADRESH,F
	banksel	ADRESL
	RRF	ADRESL,W
	banksel	HODNOTA1
	MOVWF	HODNOTA1

	MOVLW	D'1'	; provedeme A/D prevod na kanalu 1 (potenciometr P3)
	CALL	PREVOD
	banksel	ADRESH
	RRF	ADRESH,F
	banksel	ADRESL
	RRF	ADRESL,F
	banksel	ADRESH
	RRF	ADRESH,F
	banksel	ADRESL
	RRF	ADRESL,W
	banksel	HODNOTA2
	MOVWF	HODNOTA2

	GOTO	OPAKUJ	; opakujeme cely cyklus znovu


; realizace intenzity a smeru otaceni motoru 1
MOTOR1	MOVWF	MTMP	; zaloha
	BTFSS	MTMP,7	; test znamenka
	BCF	SMER1	; pokud je hodnota nezaporna, potom je smer otaceni primy
	BTFSC	MTMP,7
	BSF	SMER1	; pokud je hodnota zaporna, potom je smer otaceni opacny
	ANDLW	B'01111111' ; oriznuti "smeti"
	CALL	STRIDA1	; nastaveni stridy kanalu 1
	RETURN

; realizace intenzity a smeru otaceni motoru 2
MOTOR2	MOVWF	MTMP
	BTFSS	MTMP,7
	BCF	SMER2
	BTFSC	MTMP,7
	BSF	SMER2
	ANDLW	B'01111111'
	CALL	STRIDA2
	RETURN

; realizace stridy pomoci PWM kanalu 1
STRIDA1	MOVWF	PWM_TEMP
	RRF	PWM_TEMP,F
	BCF	CCP1CON,CCP1Y
	BTFSC	STATUS,C
	BSF	CCP1CON,CCP1Y
	RRF	PWM_TEMP,W
	BCF	CCP1CON,CCP1X
	BTFSC	STATUS,C
	BSF	CCP1CON,CCP1X
	ANDLW	B'00111111'
	MOVWF	CCPR1L
	RETURN

; realizace stridy pomoci PWM kanalu 2
STRIDA2	MOVWF	PWM_TEMP
	RRF	PWM_TEMP,F
	BCF	CCP2CON,CCP2Y
	BTFSC	STATUS,C
	BSF	CCP2CON,CCP2Y
	RRF	PWM_TEMP,W
	BCF	CCP2CON,CCP2X
	BTFSC	STATUS,C
	BSF	CCP2CON,CCP2X
	ANDLW	B'00111111'
	MOVWF	CCPR2L
	RETURN

;----------------------------------------------------------

	; deleni
DIV16U	DIV16U_M	AA,BB,CC,VD16U 

	; skripty
SKRIPT	SKRIPT_	TABULKY, PUTZNAK, DPRIKAZ, NUMFORM_, DIV16U, AA, BB, CC
	RETURN

	; obsluha displeje
	DISPLEJ	TMH,TML,DDATA,DB4,DB5,DB6,DB7,E,RW,RS

	; A/D prevod
PREVOD	N_PREV	ADTEMP,ADRESL_,ADRESH_,AD_COUNT,D'0'

;-----------------------------------------
; zobrazovaci data:

TABULKY	DATA	UVOD

UVOD	POS_D	D'0'
	DE	"OVLADANI MOTORU "
	POS_D	D'64'
	DE	"h1: "
	PRINTV	HODNOTA1,B'00111',0,0,1,1
	DE	"  "
	DE	"h2: "
	PRINTV	HODNOTA2,B'00111',0,0,1,1
	DE	"      "
	EOFS


	END
